from . import utilities
from pydantic import BaseModel
from typing import List,Optional

class Color(BaseModel):
    hex: Optional[str]
    rgba: Optional[tuple]

class ElementV2(BaseModel):
    id: Optional[str]
    name: Optional[str]
    color: Optional[str]
    icon: Optional[str]

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.icon = f"https://raw.githubusercontent.com/Mar-7th/StarRailRes/master/{self.icon}"
        self.color = Color(hex = self.color, rgba = utilities.hex_to_rgba(self.color))

class SkillV2(BaseModel):
    id: Optional[str]
    name: Optional[str]
    level: Optional[int]
    max_level: Optional[int]
    element: Optional[ElementV2]
    type: Optional[str]
    type_text: Optional[str]
    effect: Optional[str]
    effect_text: Optional[str]
    simple_desc: Optional[str]
    desc: Optional[str]
    icon: Optional[str]

class PathV2(BaseModel):
    id: Optional[str]
    name: Optional[str]
    icon: Optional[str]

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.icon = f"https://raw.githubusercontent.com/Mar-7th/StarRailRes/master/{self.icon}"

class AttributeV2(BaseModel):
    field: Optional[str]
    name: Optional[str]
    icon: Optional[str]
    value: Optional[float]
    display: Optional[str]
    percent: Optional[bool]

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.icon = f"https://raw.githubusercontent.com/Mar-7th/StarRailRes/master/{self.icon}"

class PropertyV2(BaseModel):
    type: Optional[str]
    field: Optional[str]
    name: Optional[str]
    icon: Optional[str]
    value: Optional[float]
    display: Optional[str]
    percent: Optional[bool]

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.icon = f"https://raw.githubusercontent.com/Mar-7th/StarRailRes/master/{self.icon}"

class LightConeV2(BaseModel):
    id: Optional[str]
    name: Optional[str]
    rarity: Optional[int]
    rank: Optional[int]
    level: Optional[int]
    promotion: Optional[int]
    icon: Optional[str] = "icon/light_cone/24000.png"
    preview: Optional[str]
    portrait: Optional[str]
    path: Optional[PathV2] 
    attributes: Optional[List[AttributeV2]]
    properties: Optional[List[PropertyV2]]

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.portrait = f"https://raw.githubusercontent.com/Mar-7th/StarRailRes/master/{self.portrait}"
        self.preview = f"https://raw.githubusercontent.com/Mar-7th/StarRailRes/master/{self.preview}"
        self.icon = f"https://raw.githubusercontent.com/Mar-7th/StarRailRes/master/{self.icon}"

class AffixV2(BaseModel):
    type: Optional[str]
    field: Optional[str]
    name: Optional[str]
    icon: Optional[str]
    value: Optional[float]
    display: Optional[str]
    percent: Optional[bool]

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.icon = f"https://raw.githubusercontent.com/Mar-7th/StarRailRes/master/{self.icon}"

class RelicV2(BaseModel):
    id: Optional[str]
    name: Optional[str]
    set_id: Optional[str]
    set_name: Optional[str]
    rarity: Optional[int]
    level: Optional[int]
    icon: Optional[str]
    main_affix: Optional[AffixV2]
    sub_affix: Optional[List[AffixV2]]

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.icon = f"https://raw.githubusercontent.com/Mar-7th/StarRailRes/master/{self.icon}"

class PropertyV2(BaseModel):
    type: Optional[str]
    field: Optional[str]
    name: Optional[str]
    icon: Optional[str]
    value: Optional[float]
    display: Optional[str]
    percent: Optional[bool]

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.icon = f"https://raw.githubusercontent.com/Mar-7th/StarRailRes/master/{self.icon}"

class RelicSetV2(BaseModel):
    id: str
    name: str
    desc: str
    properties: Optional[List[PropertyV2]]


class Addition(BaseModel):
    field: str
    name: str
    icon: str
    value: float
    display: str
    percent: bool

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.icon = f"https://raw.githubusercontent.com/Mar-7th/StarRailRes/master/{self.icon}"

class SkillTrees(BaseModel):
    id: Optional[int]
    level: Optional[int]
    icon: Optional[str]
    
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.icon = f"https://raw.githubusercontent.com/Mar-7th/StarRailRes/master/{self.icon}"

class CharacterData(BaseModel):
    id: Optional[str]
    name: Optional[str]
    rarity: Optional[int]
    rank: Optional[int]
    level: Optional[int]
    promotion: Optional[int]
    icon: Optional[str]
    preview: Optional[str]
    portrait: Optional[str]
    path: Optional[PathV2]
    rank_icons: Optional[list]
    element: Optional[ElementV2]
    skills: Optional[List[SkillV2]]
    skill_trees: Optional[List[SkillTrees]]
    light_cone: Optional[LightConeV2]
    relics: Optional[List[RelicV2]]
    relic_sets: Optional[List[RelicSetV2]]
    additions: Optional[List[Addition]]
    attributes: Optional[List[AttributeV2]]
    properties: Optional[List[PropertyV2]]

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.name = self.name.format(NICKNAME = "Trailblazer")
        self.icon = f"https://raw.githubusercontent.com/Mar-7th/StarRailRes/master/{self.icon}"
        self.preview = f"https://raw.githubusercontent.com/Mar-7th/StarRailRes/master/{self.preview}"
        self.portrait = f"https://raw.githubusercontent.com/Mar-7th/StarRailRes/master/{self.portrait}"
        new_rank_icons = []
        for key in self.rank_icons:
            new_rank_icons.append(f"https://raw.githubusercontent.com/Mar-7th/StarRailRes/master/{key}")
        self.rank_icons = new_rank_icons



class Avatar(BaseModel):
    id: Optional[str]
    name: Optional[str]
    icon: Optional[str]

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.icon = f"https://raw.githubusercontent.com/Mar-7th/StarRailRes/master/{self.icon}"

class SpaceInfo(BaseModel):
    pass_area_progress: Optional[int]
    light_cone_count: Optional[int]
    avatar_count: Optional[int]
    achievement_count: Optional[int]

class PlayerV2(BaseModel):
    uid: Optional[str]
    nickname: Optional[str]
    level: Optional[int]
    avatar: Optional[Avatar]
    signature: Optional[str]
    friend_count: Optional[int]
    world_level: Optional[int]
    birthday: Optional[str]
    space_info: Optional[SpaceInfo]

class StarRailApiDataV2(BaseModel):
    player: PlayerV2
    characters: List[CharacterData]


import aiohttp
import asyncio
import json
from .modalV1 import StarRailApiData
from .modalV2 import StarRailApiDataV2
from . import utilities

SUPPORTED_LANGUAGES = {
    'cht': 'cht',
    'cn': 'cn',
    'de': 'de',
    'en': 'en',
    'es': 'es',
    'fr': 'fr',
    'id': 'id',
    'jp': 'jp',
    'kr': 'kr',
    'pt': 'pt',
    'ru': 'ru',
    'th': 'th',
    'vi': 'vi'
}

API_LINK = "https://api.mihomo.me/sr_info_parsed/{uid}?lang={lang}&version=v1"
API_LINK_V2 = "https://api.mihomo.me/sr_info_parsed/{uid}?lang={lang}"
FULL_API_LINK = "https://api.mihomo.me/sr_info/{uid}?lang={lang}&version=v1"
LIGHT_CONES_LINK = "https://mana.wiki/starrail/collections/lightCones/{id}?_data=_custom%2Froutes%2F%24siteId.collections%2B%2FlightCones_.%24entryId"

async def fetch_data(url):
    async with aiohttp.ClientSession() as session:
        async with session.get(url) as response:
            data = await response.json()
            if "detail" in data:
                raise TypeError(data["detail"])
            return data

async def get_user_data(uid, lang, v):
    lang = SUPPORTED_LANGUAGES.get(lang.lower(), "en")
    if v == 1:
        url = API_LINK.format(uid=uid, lang=lang)
    else:
        url = API_LINK_V2.format(uid=uid, lang=lang)
    return await fetch_data(url)

async def get_full_user_data(uid, lang):
    lang = SUPPORTED_LANGUAGES.get(lang.lower(), "en")
    url = FULL_API_LINK.format(uid=uid, lang=lang)
    return await fetch_data(url)

async def get_light_cones_data(id):
    url = LIGHT_CONES_LINK.format(id=id)
    return await fetch_data(url)

async def get_data(lang, uid, v):
    data = await get_user_data(uid, lang, v)
    if v == 1:
        return StarRailApiData(player=data["player"], characters=data["characters"])
    return StarRailApiDataV2(player=data["player"], characters=data["characters"])

async def get_full(lang, uid,v):
    data = await get_user_data(uid, lang,v)
    for key in data["characters"]:
        if key["light_cone"] == {}:
            key["light_cone"] = None
    if v == 1:
        profile = await get_full_user_data(uid,lang)
        player_info = profile["detailInfo"]
        data["player"]["friends"] = player_info["friendCount"]
        data["player"]["worldlevel"] = player_info["worldLevel"]
        data["player"]["birthday"] = "0"#utilities.convert_date(profile["Birthday"])
        data["player"]["pass_area_progress"] = player_info["recordInfo"]["challengeInfo"]["scheduleMaxLevel"]
        data["player"]["light_cone"] = player_info["recordInfo"]["equipmentCount"]
        data["player"]["characters"] = player_info["recordInfo"]["avatarCount"]
        data["player"]["achievement"] = player_info["recordInfo"]["achievementCount"]

        return StarRailApiData(player=data["player"], characters=data["characters"])
    return StarRailApiDataV2(player=data["player"], characters=data["characters"])


    


from .src.tools import api, modalV1,utilities

class StarRailApi():
    def __init__(self,lang, v = 1) -> None:
        self.lang = lang
        if not v in [1,2]:
            v = 1
        self.v = v

    async def get_user(self,uid):
        data = await api.get_data(self.lang,uid,self.v)
        return data
    
    async def get_user_profile(self,uid):
        data = await api.get_profile(self.lang,uid)
        return data
    
    async def get_full_data(self,uid):
        data = await api.get_full(self.lang,uid,self.v)
        return data
    
    async def get_light_cone_info(self, instance):
        stats = await api.get_light_cones_data(instance.id)
        data = utilities.get_stats_light_cone(stats["defaultData"]["stats_csv"], instance.level)
        patch = stats["defaultData"]["path"]
        icon = patch["icon"]
        data["patch"] = {
            "name": patch["name"],
            "url": icon["url"]
        }
        return modalV1.LightConeInfo(**data)


# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['honkairail', 'honkairail.src', 'honkairail.src.tools']

package_data = \
{'': ['*']}

setup_kwargs = {
    'name': 'honkairail',
    'version': '1.0.2',
    'description': 'A simple and convenient model for Mihoyo API which is complemented by other resources',
    'long_description': '# HonkaiRail\n A simple and convenient model for Mihoyo API which is complemented by other resources\n\n\n## Install:\n\n```\npip install honkairail\n```\n\n## Uses:\n``` py\nfrom honkairail import starrailapi\nimport asyncio\n\n#The sample code below matches version 1 (v = 1), but you can set it to 1 or 2. The second version is newer and contains a bit more data.\nasync def main(uid,lang):\n    r = starrailapi.StarRailApi(lang, v = 1)\n    data = await r.get_full_data(uid)\n    print(data)\n\nasyncio.run(main(700649319,"en"))\n```\n\n## Usage example:\n\n```py\nfrom honkairail import starrailapi\nimport asyncio\n\n\nasync def main(uid,lang):\n    r = starrailapi.StarRailApi(lang)\n    data = await r.get_full_data(uid)\n    print("====Player====")\n    print(f"Name: {data.player.name}")\n    print(f"UID: {data.player.uid}")\n    print(f"Level: {data.player.level}")\n    print(f"World Level: {data.player.worldlevel}")\n    \n    print(f"Friends: {data.player.friends}")\n\n    print(f"Pass Area Progress: {data.player.pass_area_progress}")\n    print(f"Achievement: {data.player.achievement}")\n\n    print(f"Characters: {data.player.characters}")\n    print(f"Light Cone: {data.player.light_cone}")\n\n    print(f"Icon: {data.player.icon}")\n    print(f"Signature: {data.player.signature}")\n    print(\'\\n\\n\')\n    print("====Characters====")\n    for character in data.characters:\n        print(f"Name: {character.name} | {character.id}")\n        print(f"Rarity: {\'★\'*character.rarity}")\n        print(f"LVL: {character.level}")\n        print(f"===={character.rank_text}====")\n        for rank in character.rank_icons:\n            print(f"Icon: {rank.icon}\\nUnlock: {rank.unlock}")\n        print("====Skill====")\n        for skill in character.skill:\n            print(f"Icon: {skill.icon}\\nLVL:{skill.level}")\n        print("=============")\n        if not character.light_cone is None:\n            print(f"Light Cone: {character.light_cone.name}")\n            print(f"Rarity: {\'★\'*character.light_cone.rarity}")\n            print(f"LVL: {character.light_cone.level} | R{character.light_cone.rank}")\n            print(f"Icon: {character.light_cone.icon}")\n            print(f"Portrait: {character.light_cone.portrait}")\n            light_cone = await r.get_light_cone_info(character.light_cone)\n            print(f"ATK: {light_cone.atk} | HP: {light_cone.hp} | DEF: {light_cone.defense}")\n            print(f"Path: {light_cone.patch.name}\\nImage: {light_cone.patch.url}")\n        print("====Stats====")\n        for property in character.property:\n            if property.addition is None:\n                print(f"{property.name}: {property.base}\\nIcon: {property.icon}")\n            else:\n                print(f"{property.name}: {property.base} ({property.addition})\\n==Icon: {property.icon}")\n        print(\'\\n\\n\')\n        print("====Relic====")\n        for i in character.relic:\n            print(f"{character.relic[i].name}: {character.relic[i].level} lvl | {\'★\'*character.relic[i].rarity}")\n            print(f"{character.relic[i].main_property.name}: {character.relic[i].main_property.value}")\n            for sub_property in character.relic[i].sub_property:\n                print(f"=={sub_property.name}: {sub_property.value}\\n====Icon: {sub_property.icon}")\n            print(\'\\n\')\n        print("\\n\\n")\n\nasyncio.run(main(700649319, "en"))\n```\n\n\n### Languages Supported\n| Languege    |  Code   | Languege    |  Code   | Languege    |  Code   |\n|-------------|---------|-------------|---------|-------------|---------|\n|  English    |     en  |  русский    |     ru  |  Chinese    |    chs  |\n|  Tiếng Việt |     vi  |  ไทย        |     th  | Taiwan     |    cn  |\n|  português  |     pt  | 한국어      |     kr  | deutsch    |     de  |\n|  日本語      |     jp  | 中文        |     zh  | español    |     es  |\n|  中文        |     zh  | Indonesian |     id  | français   |     fr  |\n\n',
    'author': 'None',
    'author_email': 'None',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/DEViantUA/HonkaiRail',
    'packages': packages,
    'package_data': package_data,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



from pydantic import BaseModel
from typing import List,Optional


class Player(BaseModel):
    uid: str
    name: str
    level: int
    icon: str
    signature: str

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.icon = f"https://raw.githubusercontent.com/Mar-7th/StarRailRes/master/{self.icon}"


class RankIcon(BaseModel):
    icon: str
    unlock: bool

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.icon = f"https://raw.githubusercontent.com/Mar-7th/StarRailRes/master/{self.icon}"

class Skill(BaseModel):
    name: str
    level: int
    type: str
    icon: str
    
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.icon = f"https://raw.githubusercontent.com/Mar-7th/StarRailRes/master/{self.icon}"

class MainProperty(BaseModel):
    name: str
    value: str
    icon: str

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.icon = f"https://raw.githubusercontent.com/Mar-7th/StarRailRes/master/{self.icon}"

class SubProperty(BaseModel):
    name: str
    value: str
    icon: str

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.icon = f"https://raw.githubusercontent.com/Mar-7th/StarRailRes/master/{self.icon}"

class Relic(BaseModel):
    name: str
    rarity: int
    level: int
    main_property: MainProperty
    sub_property: List[SubProperty]
    icon: str

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.icon = f"https://raw.githubusercontent.com/Mar-7th/StarRailRes/master/{self.icon}"

class RelicSet(BaseModel):
    name: str
    icon: str
    desc: str

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.icon = f"https://raw.githubusercontent.com/Mar-7th/StarRailRes/master/{self.icon}"

class Property(BaseModel):
    name: str
    base: str
    addition: Optional[str]
    icon: str

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.icon = f"https://raw.githubusercontent.com/Mar-7th/StarRailRes/master/{self.icon}"

class PatchInfo(BaseModel):
    name: Optional[str]
    url: Optional[str]

class LightConeInfo(BaseModel):
    atk: Optional[int]
    hp: Optional[int]
    defense: Optional[int]
    patch: Optional[PatchInfo]

class LightCone(BaseModel):
    id: Optional[int]
    name: Optional[str]
    rarity: Optional[int]
    rank: Optional[int]
    level: Optional[int]
    icon: Optional[str] = "icon/light_cone/24000.png"
    portrait: Optional[str]

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.id = self.icon.split('/')[2][:-4]
        portrait = self.icon.replace("icon/light_cone/", "image/light_cone_portrait/")
        self.portrait = f"https://raw.githubusercontent.com/Mar-7th/StarRailRes/master/{portrait}"
        self.icon = f"https://raw.githubusercontent.com/Mar-7th/StarRailRes/master/{self.icon}"
        

class Character(BaseModel):
    id: Optional[str]
    name: Optional[str]
    rarity: Optional[int]
    level: Optional[int]
    rank: Optional[int]
    rank_text: Optional[str]
    rank_icons: List[RankIcon]
    preview: Optional[str]
    portrait: Optional[str]
    path: Optional[str]
    path_icon: Optional[str]
    element: Optional[str]
    element_icon: Optional[str]
    color: Optional[str]
    skill: List[Skill]
    light_cone: Optional[LightCone]
    relic: dict[str, Relic]
    relic_set: List[RelicSet]
    property: List[Property]
    
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.name = self.name.format(NICKNAME = "Trailblazer")
        self.element_icon = f"https://raw.githubusercontent.com/Mar-7th/StarRailRes/master/{self.element_icon}"
        self.path_icon = f"https://raw.githubusercontent.com/Mar-7th/StarRailRes/master/{self.path_icon}"
        self.preview = f"https://raw.githubusercontent.com/Mar-7th/StarRailRes/master/{self.preview}"
        self.portrait = f"https://raw.githubusercontent.com/Mar-7th/StarRailRes/master/{self.portrait}"



class StarRailApiData(BaseModel):
    player: Player
    characters: List[Character]

class PlayerSpaceInfo(BaseModel):
    PassAreaProgress: Optional[int]
    LightConeCount: Optional[int]
    AvatarCount: Optional[int]
    AchievementCount: Optional[int]


class PlayerDetailInfo(BaseModel):
    UID: int
    CurFriendCount: Optional[int]
    WorldLevel: Optional[int]
    Signature: Optional[str]
    NickName: Optional[str]
    Birthday: Optional[int]
    Level: Optional[int]
    PlayerSpaceInfo: Optional[PlayerSpaceInfo]


class PlayerInfo(BaseModel):
    PlayerDetailInfo: PlayerDetailInfo
  




def get_stats_light_cone(string, lvl):
    if lvl is None:
        return {"atk": 0, "hp": 0, "defense": 0}
    lines = string.split("<br/>")
    headers = lines[0].split(",")

    result_dict = {}

    for line in lines[1:]:
        values = line.split(",")
        
        if len(headers) != len(values):
            continue
        
        level = values[0]
        entry = {headers[i].lower(): int(float(values[i])) for i in range(1, len(headers))}
        if "def" in entry:
            entry["defense"] = entry.pop("def")
        
        result_dict[level] = entry

    if lvl == 0:
        lvl = 1

    return result_dict.get(str(lvl), {"atk": 0, "hp": 0, "defense": 0})

def hex_to_rgba(hex_code):
    hex_code = hex_code.strip('#')
    
    red = int(hex_code[0:2], 16)
    green = int(hex_code[2:4], 16)
    blue = int(hex_code[4:6], 16)
    
    alpha = 255 
    
    return red, green, blue, alpha


def convert_date(date):
    date = str(date)
    month = date[:-2].zfill(2)
    day = date[-2:].zfill(2)
    converted_date = f"{month}.{day}"
    return converted_date